"use strict";

var path = require('path');

var webpack = require('webpack');

var slash = require('slash2');

var pkg = require('../../package.json');

var CopyPlugin = require('copy-webpack-plugin');

var ASSET_PATH = process.env.ASSET_PATH || '/';

module.exports = function (options) {
  return {
    mode: options.mode,
    entry: options.entry,
    output: Object.assign({
      path: path.resolve(process.cwd(), pkg.name.toLocaleLowerCase()),
      publicPath: "/".concat(pkg.name.toLocaleLowerCase())
    }, options.output),
    // Merge with env dependent settings
    optimization: options.optimization,
    module: {
      rules: [{
        test: /\.jsx?$/,
        exclude: /node_modules/,
        use: {
          loader: 'babel-loader',
          options: options.babelQuery
        }
      }, {
        // Preprocess our own .css files
        // This is the place to add your own loaders (e.g. sass/less etc.)
        // for a list of loaders, see https://webpack.js.org/loaders/#styling
        test: /\.css$/,
        exclude: /node_modules/,
        use: ['style-loader', 'css-loader']
      }, {
        test: /\.less$/,
        use: [{
          loader: 'style-loader'
        }, {
          loader: 'css-loader',
          options: {
            modules: {
              // localIdentName: '[name]__[local]___[hash:base64:5]'
              // modules: true,
              getLocalIdent: function getLocalIdent(context, _, localName) {
                if (context.resourcePath.includes('node_modules') || context.resourcePath.includes('ant.design.pro.less') || context.resourcePath.includes('global.less')) {
                  return localName;
                }

                var match = context.resourcePath.match(/src(.*)/);

                if (match && match[1]) {
                  var antdProPath = match[1].replace('.less', '');
                  var arr = slash(antdProPath).split('/').map(function (a) {
                    return a.replace(/([A-Z])/g, '-$1');
                  }).map(function (a) {
                    return a.toLowerCase();
                  });
                  return "panda-pro".concat(arr.join('-'), "-").concat(localName).replace(/--/g, '-');
                }

                return localName;
              }
            }
          }
        }, {
          loader: 'less-loader',
          options: {
            javascriptEnabled: true
          }
        }]
      }, {
        test: /\.css$/,
        include: /node_modules/,
        use: ['style-loader', 'css-loader']
      }, {
        test: /\.(eot|otf|ttf|woff|woff2)$/,
        use: 'file-loader'
      }, {
        test: /\.svg$/,
        use: [{
          loader: 'svg-url-loader',
          options: {
            // Inline files smaller than 10 kB
            limit: 10 * 1024,
            noquotes: true
          }
        }]
      }, {
        test: /\.(jpg|png|gif)$/,
        use: [{
          loader: 'url-loader',
          options: {
            // Inline files smaller than 10 kB
            limit: 10 * 1024
          }
        } // {
        //   loader: 'image-webpack-loader',
        //   options: {
        //     mozjpeg: {
        //       enabled: false,
        //       // NOTE: mozjpeg is disabled as it causes errors in some Linux environments
        //       // Try enabling it in your environment by switching the config to:
        //       // enabled: true,
        //       // progressive: true,
        //     },
        //     gifsicle: {
        //       interlaced: false,
        //     },
        //     optipng: {
        //       optimizationLevel: 7,
        //     },
        //     pngquant: {
        //       quality: '65-90',
        //       speed: 4,
        //     },
        //   },
        // },
        ]
      }, {
        test: /\.html$/,
        use: 'html-loader'
      }, {
        test: /\.(mp4|webm)$/,
        use: {
          loader: 'url-loader',
          options: {
            limit: 10000
          }
        }
      }]
    },
    plugins: options.plugins.concat([// Always expose NODE_ENV to webpack, in order to use `process.env.NODE_ENV`
    // inside your code for any environment checks; Terser will automatically
    // drop any unreachable code.
    new webpack.EnvironmentPlugin({
      NODE_ENV: 'development'
    }), new webpack.DefinePlugin({
      'process.env.ASSET_PATH': JSON.stringify(ASSET_PATH)
    }), new CopyPlugin([{
      from: path.resolve(process.cwd(), 'theme'),
      to: path.resolve(process.cwd(), "".concat(pkg.name.toLocaleLowerCase(), "/theme"))
    }])]),
    resolve: {
      modules: ['node_modules', 'src'],
      extensions: ['.js', '.jsx', '.react.js'],
      mainFields: ['browser', 'jsnext:main', 'main']
    },
    devtool: options.devtool,
    target: 'web',
    // Make web variables accessible to webpack, e.g. window
    performance: options.performance || {}
  };
};